@@ -571,6 +571,24 @@ def get_mcp_disabled():
     return False
 
 
+def get_grep_output_verbose():
+    """
+    Checks puppy.cfg for 'grep_output_verbose' (case-insensitive in value only).
+    Defaults to False (concise output) if not set.
+    Allowed values for ON: 1, '1', 'true', 'yes', 'on' (all case-insensitive for value).
+
+    When False (default): Shows only file names with match counts
+    When True: Shows full output with line numbers and content
+    """
+    true_vals = {"1", "true", "yes", "on"}
+    cfg_val = get_value("grep_output_verbose")
+    if cfg_val is not None:
+        if str(cfg_val).strip().lower() in true_vals:
+            return True
+        return False
+    return False
+
+
 def get_protected_token_count():
     """
     Returns the user-configured protected token count for message history compaction.
@@ -608,7 +626,7 @@ def get_compaction_threshold():
     try:
         threshold = float(val) if val else 0.85
         # Clamp between reasonable bounds
-        return max(0.8, min(0.95, threshold))
+        return max(0.5, min(0.95, threshold))
     except (ValueError, TypeError):
         return 0.85
 
@@ -791,7 +809,6 @@ def set_diff_highlight_style(style: str):
     if style.lower() not in ["text", "highlighted"]:
         raise ValueError("diff_highlight_style must be 'text' or 'highlighted'")
     set_config_value("diff_highlight_style", style.lower())
-    _emit_diff_style_example()
 
 
 def get_diff_addition_color() -> str:
@@ -812,7 +829,6 @@ def set_diff_addition_color(color: str):
         color: Rich color markup (e.g., 'green', 'on_green', 'bright_green')
     """
     set_config_value("diff_addition_color", color)
-    _emit_diff_style_example()
 
 
 def get_diff_deletion_color() -> str:
@@ -833,11 +849,11 @@ def set_diff_deletion_color(color: str):
         color: Rich color markup (e.g., 'orange1', 'on_bright_yellow', 'red')
     """
     set_config_value("diff_deletion_color", color)
-    _emit_diff_style_example()
 
 
 def _emit_diff_style_example():
     """Emit a small diff example showing the current style configuration."""
+
     try:
         from code_puppy.messaging import emit_info
         from code_puppy.tools.file_modifications import _colorize_diff
