@@ -1,6 +1,7 @@
 import argparse
 import asyncio
 import os
+import platform
 import subprocess
 import sys
 import time
@@ -29,7 +30,6 @@ from code_puppy.config import (
 )
 from code_puppy.http_utils import find_available_port
 from code_puppy.messaging import emit_info
-from code_puppy.session_storage import restore_autosave_interactively
 from code_puppy.tools.common import console
 
 # message_history_accumulator and prune_interrupted_tool_calls have been moved to BaseAgent class
@@ -170,29 +170,33 @@ async def main():
             sys.exit(1)
     from code_puppy.messaging import emit_system_message
 
-    # Badass Code Puppy intro with pyfiglet and blue-to-green gradient
-    try:
-        import pyfiglet
+    # Show the awesome Code Puppy logo only in interactive mode (never in TUI mode)
+    # Always check both command line args AND runtime TUI state for safety
+    if args.interactive and not args.tui and not args.web and not is_tui_mode():
+        try:
+            import pyfiglet
 
-        intro_lines = pyfiglet.figlet_format("CODE PUPPY", font="ansi_shadow").split(
-            "\n"
-        )
+            intro_lines = pyfiglet.figlet_format(
+                "CODE PUPPY", font="ansi_shadow"
+            ).split("\n")
 
-        # Simple blue to green gradient (top to bottom)
-        gradient_colors = ["bright_blue", "bright_cyan", "bright_green"]
+            # Simple blue to green gradient (top to bottom)
+            gradient_colors = ["bright_blue", "bright_cyan", "bright_green"]
+            emit_system_message("\n\n")
 
-        # Apply gradient line by line
-        for line_num, line in enumerate(intro_lines):
-            if line.strip():
-                # Use line position to determine color (top blue, middle cyan, bottom green)
-                color_idx = min(line_num // 2, len(gradient_colors) - 1)
-                color = gradient_colors[color_idx]
-                emit_system_message(f"[{color}]{line}[/{color}]")
-            else:
-                emit_system_message("")
-
-    except ImportError:
-        emit_system_message("🐶 Code Puppy is Loading...")
+            lines = []
+            # Apply gradient line by line
+            for line_num, line in enumerate(intro_lines):
+                if line.strip():
+                    # Use line position to determine color (top blue, middle cyan, bottom green)
+                    color_idx = min(line_num // 2, len(gradient_colors) - 1)
+                    color = gradient_colors[color_idx]
+                    lines.append(f"[{color}]{line}[/{color}]")
+                else:
+                    lines.append("")
+            emit_system_message("\n".join(lines))
+        except ImportError:
+            emit_system_message("🐶 Code Puppy is Loading...")
 
     available_port = find_available_port()
     if available_port is None:
@@ -295,6 +299,10 @@ async def main():
 
     # Initialize DBOS if not disabled
     if get_use_dbos():
+        # Append a Unix timestamp in ms to the version for uniqueness
+        dbos_app_version = os.environ.get(
+            "DBOS_APP_VERSION", f"{current_version}-{int(time.time() * 1000)}"
+        )
         dbos_config: DBOSConfig = {
             "name": "dbos-code-puppy",
             "system_database_url": DBOS_DATABASE_URL,
@@ -305,7 +313,7 @@ async def main():
             "log_level": os.environ.get(
                 "DBOS_LOG_LEVEL", "ERROR"
             ),  # Default to ERROR level to suppress verbose logs
-            "application_version": current_version,  # Match DBOS app version to Code Puppy version
+            "application_version": dbos_app_version,  # Match DBOS app version to Code Puppy version
         }
         try:
             DBOS(config=dbos_config)
@@ -314,7 +322,7 @@ async def main():
             emit_system_message(f"[bold red]Error initializing DBOS:[/bold red] {e}")
             sys.exit(1)
     else:
-        emit_system_message("DBOS is disabled. Running without durable execution.")
+        pass
 
     global shutdown_flag
     shutdown_flag = False
@@ -371,21 +379,22 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
     display_console = message_renderer.console
     from code_puppy.messaging import emit_info, emit_system_message
 
-    emit_info("[bold green]Code Puppy[/bold green] - Interactive Mode")
-    emit_system_message("Type '/exit' or '/quit' to exit the interactive mode.")
-    emit_system_message("Type 'clear' to reset the conversation history.")
+    emit_system_message(
+        "[dim]Type '/exit' or '/quit' to exit the interactive mode.[/dim]"
+    )
+    emit_system_message("[dim]Type 'clear' to reset the conversation history.[/dim]")
     emit_system_message("[dim]Type /help to view all commands[/dim]")
     emit_system_message(
-        "Type [bold blue]@[/bold blue] for path completion, or [bold blue]/m[/bold blue] to pick a model. Toggle multiline with [bold blue]Alt+M[/bold blue] or [bold blue]F2[/bold blue]; newline: [bold blue]Ctrl+J[/bold blue]."
+        "[dim]Type [bold blue]@[/bold blue] for path completion, or [bold blue]/model[/bold blue] to pick a model. Toggle multiline with [bold blue]Alt+M[/bold blue] or [bold blue]F2[/bold blue]; newline: [bold blue]Ctrl+J[/bold blue].[/dim]"
     )
     emit_system_message(
-        "Press [bold red]Ctrl+C[/bold red] during processing to cancel the current task or inference."
+        "[dim]Press [bold red]Ctrl+C[/bold red] during processing to cancel the current task or inference. Use [bold red]Ctrl+X[/bold red] to interrupt running shell commands.[/dim]"
     )
     emit_system_message(
-        "Use [bold blue]/autosave_load[/bold blue] to manually load a previous autosave session."
+        "[dim]Use [bold blue]/autosave_load[/bold blue] to manually load a previous autosave session.[/dim]"
     )
     emit_system_message(
-        "Use [bold blue]/diff[/bold blue] to configure diff highlighting colors for file changes."
+        "[dim]Use [bold blue]/diff[/bold blue] to configure diff highlighting colors for file changes.[/dim]"
     )
     try:
         from code_puppy.command_line.motd import print_motd
@@ -395,9 +404,6 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
         from code_puppy.messaging import emit_warning
 
         emit_warning(f"MOTD error: {e}")
-    from code_puppy.messaging import emit_info
-
-    emit_info("[bold cyan]Initializing agent...[/bold cyan]")
 
     # Initialize the runtime agent manager
     if initial_command:
@@ -485,7 +491,7 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
         current_agent = get_current_agent()
         user_prompt = current_agent.get_user_prompt() or "Enter your coding task:"
 
-        emit_info(f"[bold blue]{user_prompt}[/bold blue]")
+        emit_info(f"[dim][bold blue]{user_prompt}\n[/bold blue][/dim]")
 
         try:
             # Use prompt_toolkit for enhanced input with path completion
@@ -540,7 +546,9 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
             new_session_id = finalize_autosave_session()
             agent.clear_message_history()
             emit_warning("Conversation history cleared!")
-            emit_system_message("The agent will not remember previous interactions.\n")
+            emit_system_message(
+                "[dim]The agent will not remember previous interactions.[/dim]"
+            )
             emit_info(f"[dim]Auto-save session rotated to: {new_session_id}[/dim]")
             continue
 
@@ -564,7 +572,65 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
                 if command_result == "__AUTOSAVE_LOAD__":
                     # Handle async autosave loading
                     try:
-                        await restore_autosave_interactively(Path(AUTOSAVE_DIR))
+                        # Check if we're in a real interactive terminal
+                        # (not pexpect/tests) - TUI requires proper TTY
+                        use_tui = sys.stdin.isatty() and sys.stdout.isatty()
+
+                        # Allow environment variable override for tests
+                        if os.getenv("CODE_PUPPY_NO_TUI") == "1":
+                            use_tui = False
+
+                        if use_tui:
+                            # Use new TUI picker for interactive sessions
+                            from code_puppy.agents.agent_manager import (
+                                get_current_agent,
+                            )
+                            from code_puppy.command_line.autosave_menu import (
+                                interactive_autosave_picker,
+                            )
+                            from code_puppy.config import (
+                                set_current_autosave_from_session_name,
+                            )
+                            from code_puppy.messaging import (
+                                emit_error,
+                                emit_success,
+                                emit_warning,
+                            )
+                            from code_puppy.session_storage import (
+                                load_session,
+                                restore_autosave_interactively,
+                            )
+
+                            chosen_session = await interactive_autosave_picker()
+
+                            if not chosen_session:
+                                emit_warning("Autosave load cancelled")
+                                continue
+
+                            # Load the session
+                            base_dir = Path(AUTOSAVE_DIR)
+                            history = load_session(chosen_session, base_dir)
+
+                            agent = get_current_agent()
+                            agent.set_message_history(history)
+
+                            # Set current autosave session
+                            set_current_autosave_from_session_name(chosen_session)
+
+                            total_tokens = sum(
+                                agent.estimate_tokens_for_message(msg)
+                                for msg in history
+                            )
+                            session_path = base_dir / f"{chosen_session}.pkl"
+
+                            emit_success(
+                                f"✅ Autosave loaded: {len(history)} messages ({total_tokens} tokens)\n"
+                                f"📁 From: {session_path}"
+                            )
+                        else:
+                            # Fall back to old text-based picker for tests/non-TTY environments
+                            await restore_autosave_interactively(Path(AUTOSAVE_DIR))
+
                     except Exception as e:
                         from code_puppy.messaging import emit_error
 
@@ -811,6 +877,15 @@ def main_entry():
         if get_use_dbos():
             DBOS.destroy()
         return 0
+    finally:
+        # Reset terminal on Unix-like systems (not Windows)
+        if platform.system() != "Windows":
+            try:
+                # Reset terminal to sanity state
+                subprocess.run(["reset"], check=True, capture_output=True)
+            except (subprocess.CalledProcessError, FileNotFoundError):
+                # Silently fail if reset command isn't available
+                pass
 
 
 if __name__ == "__main__":
